#include <usb.h>
#include <stdarg.h>

static int idVendor=0x3f0;
static int idProduct1=0x805;
static int idProduct2=0x705;

extern int usb_set_configuration(struct usb_dev_handle *dev, int configuration);
extern int usb_claim_interface(struct usb_dev_handle *dev, int interface);

void
usb_error( const char *format, ...)
{
  va_list args;

  fputs( "status: ", stderr );

  va_start (args,format);
  vfprintf( stderr, format, args);
  va_end (args);

  fprintf( stderr,"\nusb error: %s\n", usb_strerror());
}

void
message( const char *format, ...)
{
  va_list args;

  va_start (args,format);
  vfprintf( stderr, format, args);
  va_end (args);

  fprintf( stderr,"\n" );
}


#ifndef USB_OK
#define USB_OK 1
#endif

#ifndef USB_DIR_IN
#define USB_DIR_IN 0x80
#endif

#ifndef USB_DIR_OUT
#define USB_DIR_OUT 0x00
#endif


usb_dev_handle *
open_device( struct usb_device *device )
{
  usb_dev_handle *handle;

  message( "opening device %s on bus %s",
	   device->filename,
	   device->bus->dirname
	   );

  handle = usb_open(device);

  if ( !handle )
    {
      message( "open failed" );
      return 0;
    }

  return handle;
}

struct usb_dev_handle *
scan_bus( struct usb_bus* bus )
{
  struct usb_device* roottree = bus->devices;
  struct usb_device *device;

  struct usb_dev_handle * next;
  struct usb_bus * nextbus;

  message( "scanning bus %s", bus->dirname );


  for( device = roottree;device;device=device->next)
    {
      if ( ((device->descriptor.idVendor == idVendor ) ) &&
	   ((device->descriptor.idProduct == idProduct2 ) || (device->descriptor.idProduct == idProduct1 ) ) )
	{
	  usb_dev_handle *device_handle;
	  message( "found device %s on bus %s (idVendor 0x%x idProduct 0x%x)",
		   device->filename,
		   device->bus->dirname,
		   device->descriptor.idVendor,
		   device->descriptor.idProduct );

	  if ( (device_handle = open_device(device)) ){
	     if (usb_claim_interface(device_handle, 0) < 0)
	     {
               usb_close(device_handle);
	       return 0;

             } else{
                message( "Device %s on bus, %s Vendor 0x%x Product 0x%x claimed.\n",device->filename,device->bus->dirname,device->descriptor.idVendor,device->descriptor.idProduct);
	       return device_handle;
             };
      }

    message( "continuing to scan bus %s", bus->dirname );
    }
    else
      message( "device %s on bus %s does not match",
    device->filename,
    device->bus->dirname);
  }
  nextbus=bus->next;
  if (!nextbus==NULL) {next = scan_bus(nextbus); return next; }

  return NULL;
}

int read_usb(struct usb_dev_handle *device,int ep,char *data,int size,int timeout)
{ //Writes out size elements from buf starting from reg
  int read;

#ifdef USBDEBUG
  char *tmp=data;
  int tmp_size=size;


    fprintf(stderr,"transfer type=bulk size=%d ep=%x dir=IN\n",size,ep);
#endif

  if ( (read = usb_bulk_read(device, ep, data, size, timeout)) !=size )
	    {
        usb_error( "problem doing read; only %d read from %d", (int)read, (int)size );

        return(read);
      }
  else
  {
#ifdef USBDEBUG
    while(tmp_size--) fprintf(stderr,"%02hhx ",*tmp++);
    fprintf(stderr,"\n");
#endif
    return(0);
  };
};

int write_usb(struct usb_dev_handle *device,int ep,char *data,int size,int timeout)
{
    int err;
#ifdef USBDEBUG
    char *tmp=data;
    int tmp_size=size;

    fprintf(stderr,"transfer type=bulk size=%d ep=%x dir=OUT\n",size,ep);
    while(tmp_size--) fprintf(stderr,"%02hhx ",*tmp++);
    fprintf(stderr,"\n");
#endif

	  if ( (err=usb_bulk_write(device, ep, data, size, timeout)) <0 )
	    {
	      usb_error( "problem doing write %d",err );
        return(0);
      } else
        return(err);
}
